﻿using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Data;
using AttachedBehaviors.Framework;

namespace AttachedBehaviors.Usage
{
	public static class BooleanVisibility
	{
		public static readonly DependencyProperty ValueProperty = DependencyProperty.RegisterAttached(
			"Value",
			typeof(bool),
			typeof(BooleanVisibility),
			new PropertyMetadata(true, OnVisibilityChanged));

		public static readonly DependencyProperty WhenTrueProperty = DependencyProperty.RegisterAttached(
			"WhenTrue",
			typeof(Visibility),
			typeof(BooleanVisibility),
			new PropertyMetadata(Visibility.Visible, OnVisibilityChanged));

		public static readonly DependencyProperty WhenFalseProperty = DependencyProperty.RegisterAttached(
			"WhenFalse",
			typeof(Visibility),
			typeof(BooleanVisibility),
			new PropertyMetadata(Visibility.Collapsed, OnVisibilityChanged));

		private static readonly AttachedBehavior Behavior =
			AttachedBehavior.Register(host => new BooleanVisibilityBehavior(host));

		private static void OnVisibilityChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			Behavior.Update(d);
		}

		public static void SetValue(UIElement uiElement, bool value)
		{
			Contract.Requires(uiElement != null);

			uiElement.SetValue(ValueProperty, value);
		}

		public static bool GetValue(UIElement uiElement)
		{
			Contract.Requires(uiElement != null);

			return (bool) uiElement.GetValue(ValueProperty);
		}

		public static void SetWhenTrue(UIElement uiElement, Visibility visibility)
		{
			Contract.Requires(uiElement != null);

			uiElement.SetValue(WhenTrueProperty, visibility);
		}

		public static Visibility GetWhenTrue(UIElement uiElement)
		{
			Contract.Requires(uiElement != null);

			return (Visibility) uiElement.GetValue(WhenTrueProperty);
		}

		public static void SetWhenFalse(UIElement uiElement, Visibility visibility)
		{
			Contract.Requires(uiElement != null);

			uiElement.SetValue(WhenFalseProperty, visibility);
		}

		public static Visibility GetWhenFalse(UIElement uiElement)
		{
			Contract.Requires(uiElement != null);

			return (Visibility) uiElement.GetValue(WhenFalseProperty);
		}

		private sealed class BooleanVisibilityBehavior : Behavior<UIElement>
		{
			internal BooleanVisibilityBehavior(DependencyObject host) : base(host)
			{
				// Does not propagate external changes in visibility to binding source - that will be
				// covered in a future post
			}

			protected override void Update(UIElement host)
			{
				host.Visibility = GetValue(host) ? GetWhenTrue(host) : GetWhenFalse(host);
			}
		}
	}
}